/*
	pp_skeleton.cmd v1.0
	There is no copyright on this one -- use it, abuse it, make it your own.

	This is a skeleton post-processing script for SmartBackup version 1.0.  It provides
	the basic needs of any post-processing script so that users can begin to build their
	own scripts to suit their unique needs.  If you need to backup to a Zip drive, a DVD
	recorder or a tape backup system, this skeleton script will get you started.

	Further examples of manipulating the data of SmartBackup project files can be found in
	the included fully functional post-processing scripts:  pp_checksize.cmd, pp_iso.cmd
	and pp_zip2iso.cmd.  The potential is limited only by your imagination and ability to
	write Rexx scripts.

	Beginners should only need to edit the two sections labeled USER SECTION #1 and USER
	SECTION #2.  The rest of the code is generic to handle the output data from SmartBackup
	and to prepare a "work file" (also referred to as ListFile) so the original output
	data is not changed/damaged and can be re-used if necessary.  However you are free to
	modify this skeleton script in any way you see fit.
*/

/* Get the command line parameter (the number of project files to process). */
PARSE ARG Project.0

/* Find the system %TEMP% directory so we can be sure to locate the project files. */
TempDir = Value('TEMP', , 'OS2ENVIRONMENT')
IF TempDir = '' THEN
	TempDir = Value('TMP', , 'OS2ENVIRONMENT')

/* Create each backup disc in turn. */
DO Num = 1 TO Project.0
	/* Set the name of the input project file. */
	ProjectFile = TempDir||'\sb_project.'||Num
	IF Stream(ProjectFile, 'c', 'query exists') = '' THEN
		LEAVE

	/* Open input project file for reading only. */
	RC = Stream(ProjectFile, 'c', 'open read')

	/* Read global configuration file because each project file could have its own configuration file. */
	CALL ReadConf

	/* We need to modify the input project file so we'll create a modified copy. */
	ListFile = ProjectFile||'.list'

	/* Open output project file for writing - to avoid problems, delete it if it already exists. */
	IF Stream(ListFile, 'c', 'query exists') <> '' THEN
		DO
			RC = SysFileDelete(ListFile)
			IF RC <> 0 THEN
				DO
					SAY 'Unable to delete existing temp file' ListFile '(rc =' RC||').'
					SAY 'I will now exit.  Please manually delete' ListFile 'and run me again.'
					EXIT
				END
		END
	RC = Stream(ListFile, 'c', 'open write')

	/* Copy the input project to the output project, removing file size and unit of measure from each line. */
	/* We store the size and unit of measure in variables in case we do need to use them.                   */
	DO While Lines(ProjectFile) > 0
		InLine = Linein(ProjectFile, , 1)
		PARSE VALUE InLine WITH Size '09'x Unit '09'x InFile

		/* If file needs to be byte split, do it now.  This allows files which are larger than a disc */
		/* to be saved in chunks to consecutive discs.                                                */
		IF (Stream(InFile, 'c', 'query exists') = '') & (Right(InFile, 4) = '.001') THEN
			DO
				/* THIS NEXT LINE IS VERY IMPORTANT!                                                   */
				/* The single largest file goes on disc 1, so the first chunk of that will be the full */
				/* disc size.  Later on we could find a chunk .002 before we have found the chunk .001 */
				/* for the same file and we'll need to know the full disc size to do proper splitting. */
				DiscSize = Size
				CALL bytesplit '-S -C' Size '-U' Unit '-I "'||Substr(InFile, 1, Length(InFile) - 4)||'"'
			END
		/* Here's where we handle a chunk .002 if we've not found its chunk .001 yet. */
		IF (Stream(InFile, 'c', 'query exists') = '') & (Substr(InFile, Length(InFile) - 3, 1) = '.') & (Verify(Right(InFile, 3), '0123456789') = 0) THEN
			IF Strip(Right(InFile, 3), 'L', '0') > 1 THEN
				CALL bytesplit '-S -C' DiscSize '-U' Unit '-I "'||Substr(InFile, 1, Length(InFile) - 4)||'"'

		CALL MakeOutLine
		CALL Lineout Listfile, OutLine
	END

	/* Close both files. */
	RC = Stream(ListFile, 'c', 'close')
	RC = Stream(ProjectFile, 'c', 'close')

	CALL MakeBackup

	/* Cleanup/delete temporary files. */
	CALL CleanUp

	/* Prompt user for next disc in a multiple-disc project. */
	IF Project.0 > Num THEN
		DO
			SAY 'Time to remove that disc and add the next ('||Num+1||'). Press ENTER when ready.'
			PAUSE
		END
END

EXIT


ReadConf:
	/* Read first line of project file to find which configuration file we're supposed to use. */
	ConfPath = Linein(ProjectFile, , 1)

	/* Make sure the first line uses the correct syntax, otherwise the file might not be a SmartBackup project. */
	IF Translate(Left(ConfPath, Pos('=', ConfPath) - 1)) = 'SBCONF' THEN
		DO
			SBConf = Substr(ConfPath, Pos('=', ConfPath) + 1)
			/* If the specified configuration file does not exist, prompt user for a better one. */
			IF Stream(SBConf, 'c', 'query exists') = '' THEN
				DO UNTIL Stream(SBConf, 'c', 'query exists') <> ''
					SAY 'Configuration file' SBConf 'cannot be found.'
					SAY 'Enter the full path and filename of the configuration file to use.'
					PARSE PULL SBConf
				END
		END
	ELSE
		DO
			SAY 'Invalid project file. Cannot find SBConf= on line 1.'
			/* Close input project file and exit. */
			RC = Stream(ProjectFile, 'c', 'close')
			EXIT
		END

	/* Read global configuration file into variables. */
	RC = Stream(SBConf, 'c', 'open read')
	DO WHILE Lines(SBConf) > 0
		InLine = Linein(SBConf, , 1)
		/* If the line is blank or a Rexx-style comment, ignore it and move on. */
		IF Left(InLine, 2) = '/*' | Left(InLine, 2) = '*/' | InLine = '' THEN
			ITERATE
		ELSE
			SELECT
				/*****************************************************************************/
				/* USER SECTION #1                                                           */
				/*****************************************************************************/
				/* If we found the configuration file, read in the variables we need to use. */
				/* NOTE:  This is another section you will need to edit.  You could read in  */
				/* every variable from the configuration file, but I suggest only getting    */
				/* those you actually need in this script and ignore all others.             */
				/*****************************************************************************/
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'OUTPUTDIR' THEN
					OutputDir = Substr(InLine, Pos('=', InLine) + 1)
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'VOLUMEID' THEN
					VolumeID = Substr(InLine, Pos('=', InLine) + 1)
				OTHERWISE
					NOP
			END
	END
	RC = Stream(SBConf, 'c', 'close')
RETURN


MakeBackup:
	/********************************************************************/
	/* USER SECTION #2                                                  */
	/********************************************************************/
	/* Here is where the majority of your own programming will reside.  */
	/* Of course you may edit other parts of this skeleton script, it   */
	/* is recommended that beginners try to do as much as they can here */
	/* and leave the rest alone.                                        */
	/********************************************************************/
RETURN


CleanUp:
	RC = SysFileDelete(ListFile)
RETURN

