/*
	SmartBackup
	(c) 2003 by Don K. Eitner
*/

/* Initialize Rexx utility functions */
CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
CALL SysLoadFuncs

SIGNAL ON NOTREADY
CALL ON HALT

/* Setup some directories we'll be using */
PARSE SOURCE . . ProgDir
ProgDir = Filespec('d', ProgDir)||Filespec('p', ProgDir)
ProgDir = Left(ProgDir, Lastpos('\', ProgDir) - 1)
CurDir = Directory()
TempDir = Value('TEMP', , 'OS2ENVIRONMENT')
IF TempDir = '' THEN
	TempDir = Value('TMP', , 'OS2ENVIRONMENT')

/* Set initial global variables */
ProgName = 'SmartBackup'
ProgVersion = 1.1
NUMERIC DIGITS 9
TotalSize = 0
Num = 0
DirNum = 0
Files.0 = 0
Project.0 = 0
NumFiles = 0
CheckDirs = 1
ExcludesFile = ProgDir||'\excludes.txt'
SBConf = ''
PostProcess = ''

/* We will be using a function only available to OS/2 Warp 4 fp13 and higher, so be sure we have that */
IF RxFuncQuery('SysStemSort') = 1 THEN
	DO
		SAY ProgName 'requires the Rexx utility library found in eComStation and OS/2'
		SAY 'Warp 4 fixpack 13 and later.  You do not seem to have this.'
		EXIT
	END

CALL ClearQ

/* Similarly, we will need my ByteSplit.cmd version 0.8 or higher */
'@bytesplit.cmd -v | RXQUEUE'
Pull BSVersion
IF BSVersion < 0.8 THEN
	DO
		SAY 'Unable to locate ByteSplit.cmd version 0.8 or higher.  Cannot continue.'
		EXIT
	END

/* Setup ANSI codes for display properties */
CALL AnsiSetup

/* Read default values from global configuration file */
CALL ReadConf

/* Now read input command line */
PARSE ARG CmdLine
CALL ParseInput

/* Do last minute global variables setup */
SELECT
	WHEN Unit = 'MB' THEN
		Precision = 2
	WHEN Unit = 'KB' THEN
		Precision = 1
	WHEN Unit = 'B' THEN
		Precision = 0
	OTHERWISE
		NOP
END

IF Translate(Left(Recurse, 1)) = 'N' | Recurse = 0 THEN
	Recurse = 0
ELSE
	Recurse = 1

NumBefore = 15 - (3 * (1 + Precision))
NumAfter = 3 * (Precision - 1)

/* Get all files from the directory tree, or give error if unable to */
SAY 'Retrieving list of files to backup...'

DO ThisDirNum = 1 TO DirNum
	ThisDir = InputDir.ThisDirNum
	NewDir = Directory(ThisDir)
	IF NewDir = '' THEN
		DO
			SAY LRed||'Cannot continue, cannot access' ThisDir||Non
			NewDir = Directory(CurDir)
			EXIT
		END
	NewDir = Directory(CurDir)

	IF Recurse = 1 THEN
		DO
			IF Right(ThisDir, 1) = '\' THEN
				ThisDir = Left(ThisDir, Length(ThisDir) - 1)
			RC = SysFileTree(ThisDir||'\', 'File', 'FS')
		END
	ELSE
		DO
			IF Right(ThisDir, 1) = '\' THEN
				ThisDir = Left(ThisDir, Length(ThisDir) - 1)
			RC = SysFileTree(ThisDir||'\', 'File', 'F')
			NewDir = Directory(CurDir)
		END

     Position = Files.0 + 1
	IF File.0 <> 0 THEN
		RC = SysStemCopy('File.', 'Files.', , Position, , 'I')
	DROP File
END

RC = SysStemCopy('Files.', 'File.')
DROP Files

/* Now that we have the complete file list, remove the ones we don't want */
CALL DiminishList

/* Determine whether we can fit all files on a single disc or if multiple discs will be needed */
/* Sort files in directory tree by size, largest first */
/* Could use a Heap Sort routine, but Rexx since Warp 4 fp13 has a built-in Quick Sort */
SAY 'Sorting input files in by size...'
RC = SysStemSort('File.', 'D', 'I',,, 20, 28)

/* If user specified minimum or maximum file sizes, honor their wishes now */
IF Datatype(MinSize) = 'NUM' & MinSize > 0 THEN
	DO
		X = File.0
		DO UNTIL Size >= MinSize
			/* step backward thru File. until we find a file not smaller than MinSize; delete everything after */
			X = X - 1
			PARSE VAR File.X Date Time Size Attribs Filename
			Size = Size / 1024**Precision
		END
		RC = SysStemDelete('File.', X + 1, File.0 - X)
	END
IF Datatype(MaxSize) = 'NUM' & MaxSize >= MinSize THEN
	DO
		Y = 0
		DO UNTIL Size <= MaxSize
			/* step forward thru File. until we find a file not larger than MaxSize; delete everything before */
			Y = Y + 1
			PARSE VAR File.Y Date Time Size Attribs Filename
			Size = Size / 1024**Precision
		END
		IF Y > 1 THEN
			RC = SysStemDelete('File.', 1, Y - 1)
	END

SAY 'Checking if all files can fit a single disc of size' DiscSize Unit||'...'
DO Num = 1 TO File.0
	PARSE VAR File.Num Date Time Size Attribs Filename
	TotalSize = TotalSize + Size
END

/* Precision of 0 = byte level, 1 = kilobyte level, 2 = megabyte level. Default is 1 */
TotalSize = TotalSize / (1024**Precision)
TotalSize = Strip(Format(TotalSize, NumBefore, NumAfter))
SAY File.0 'files to be backed up.  Total size =' TotalSize Unit

IF TotalSize <= DiscSize THEN
	SAY LGre||'All files should fit on a single disc!'||Non

/* Now make the backup project files */
CALL RoundRobin

SAY 'Backing up all these files should require' Project.0 DiscSize Unit 'disc(s).'

SAY
SAY 'File lists have been created as sb_project.# where # indicates the disc number'
SAY 'within the backup project.'
SAY

DO Project = 1 TO Project.0
	RC = Stream(TempDir||'\sb_project.'||Project, 'c', 'close')
END

/* Do post-processing (eg. create Data-CD-Creator folders or run mkisofs and pipe it to cdrecord, etc) */
CALL PostProcess

EXIT


AnsiSetup:
	Esc=d2c(27)d2c(91)

	/* Attributes */
	Hig=Esc'1m'
	Non=Esc'0m'

	/* Foreground Colors */
	Bla=Esc'30m'
	Red=Esc'31m'
	Gre=Esc'32m'
	Bro=Esc'33m'
	Blu=Esc'34m'
	Pur=Esc'35m'
	Tur=Esc'36m'
	Whi=Esc'37m'
	LBla=Hig||Bla
	LRed=Hig||Red
	LGre=Hig||Gre
	LBro=Hig||Bro
	LBlu=Hig||Blu
	LPur=Hig||Pur
	LTur=Hig||Tur
	LWhi=Hig||Whi

	/* Background Colors */
	BBla=Esc'40m'
	BRed=Esc'41m'
	BGre=Esc'42m'
	BBro=Esc'43m'
	BBlu=Esc'44m'
	BCya=Esc'45m'
	BTur=Esc'46m'
	BWhi=Esc'47m'
RETURN


ParseInput:
	IF CmdLine = '' THEN
		CALL GiveHelp

	DO WHILE CmdLine <> ''
		PARSE VAR CmdLine CL1 CmdLine
		IF Left(CL1, 1) = '/' | Left(CL1, 1) = '-' THEN
			DO
				CL1 = Translate(CL1)
				SELECT
					WHEN Substr(CL1, 2) = 'S' THEN   /* get disc size */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							DiscSize = Translate(CL1)
						END
					WHEN Substr(CL1, 2) = 'U' THEN   /* get unit of measure */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							Unit = Translate(CL1)
						END
					WHEN Substr(CL1, 2) = 'MIN' THEN   /* get minimum allowed file size */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							MinSize = Translate(CL1)
						END
					WHEN Substr(CL1, 2) = 'MAX' THEN   /* get maximum allowed file size */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							MaxSize = Translate(CL1)
						END
					WHEN Substr(CL1, 2) = 'R' THEN   /* inclue sub-directories? */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							Recurse = CL1
							IF Recurse = 1 or Translate(Recurse) = 'YES' THEN
								Recurse = 'YES'
						END
					WHEN Substr(CL1, 2) = 'E' THEN   /* include empty directories? */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							EmptyDirs = CL1
							IF EmptyDirs = 1 or Translate(EmptyDirs) = 'YES' THEN
								EmptyDirs = 'YES'
						END
					WHEN Substr(CL1, 2) = 'C' THEN   /* get configuration file to use */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							SBConf = CL1
						END
					WHEN Substr(CL1, 2) = 'P' THEN   /* get post-processing script */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							PostProcess = CL1
						END
					WHEN Substr(CL1, 2) = 'I' THEN   /* get input directory */
						DO WHILE Left(CmdLine, 1) <> '-' & Left(CmdLine, 1) <> '/' & CmdLine <> ''
							DirNum = DirNum + 1
							IF Left(CmdLine, 1) = '"' THEN
								DO
									PARSE VAR CmdLine WITH '"' CL1 '"' CmdLine
									InputDir.DirNum = CL1
								END
							ELSE
								DO
									PARSE VAR CmdLine CL1 CmdLine
									InputDir.DirNum = CL1
								END
							CmdLine = Strip(CmdLine, 'L', ' ')
						END
					WHEN (Substr(CL1, 2) = 'V') | (Substr(CL1, 2) = 'VERSION') THEN   /* show program version */
						DO
							SAY ProgVersion
							EXIT
						END
					OTHERWISE
						CALL GiveHelp
				END
			END
		ELSE
			DO
				CALL GiveHelp
				EXIT
			END
	END
RETURN


GiveHelp:
	CALL SysCls
	SAY LTur||ProgName ProgVersion '(c) 2003 by Don Eitner'||Non
	SAY LGre||'Usage:'||Non
	SAY '  '||ProgName' -I InputDir -S DiscSize [options]'
	SAY
	SAY 'InputDir must be enclosed in quotation marks (") if it contains spaces.'
	SAY 'DiscSize specifies the maximum capacity of each output disc.'
	SAY
	SAY LGre||'Available Options:'||Non
	SAY '-U Unit = the unit of measure (MB, KB or B) for DiscSize, MinSize & MaxSize.'
	SAY '  Default is MB.'
	SAY '-R Recurse = if NO, does not find files in sub-directories.  Default is YES.'
	SAY '-E EmptyDirs = if YES, includes empty sub-directories  Default is NO.'
	SAY '-MIN MinSize = specify minimum file size to include.  Default is 0.'
	SAY '-MAX MaxSize = specify maximum file size to include.  Default is infinite.'
	SAY '-C ConfigFile = specify name of configuration file to read.'
	SAY '-P Process = specify name of post-processing script to run.  If not given,'
	SAY '  '||ProgName 'will prompt you.'
	SAY '-V = show' ProgName 'version.'
	SAY '-? = show this help screen.'
	SAY
	SAY LTur||'NOTE: Most CDs hold slightly more or less than their stated capacity.  If you'
	SAY 'are unable to copy a full' ProgName 'project to your discs, re-run' ProgName
	SAY 'with a lower DiscSize value (eg. 630 or 670, etc.).'||Non
EXIT


ReadConf:
	/* If configuration file was not specified, use default name */
	IF SBConf = '' THEN
		DO
			SBConfGiven = 0
			SBConf = 'smartbackup.conf'
		END
	ELSE
		/* If configuration file was specified but only as a filename, note that something was given */
		IF Filespec('p', SBConf) = '' THEN
			SBConfGiven = 1
		ELSE
			/* If configuration file was specified with a full path name, note that we got it all */
			SBConfGiven = 2

	/* If we got it all, be sure the file exists */
	IF SBConfGiven = 2 THEN
		DO
			IF Stream(SBConf, 'c', 'query exists') <> '' THEN
				Found = 1
		END
	ELSE
		/* If we got nothing or only got a filename, be sure we can find it */
		DO UNTIL Found = 1 | CheckDirs > 4
			SELECT
				WHEN CheckDirs = 1 THEN
					CheckDir = ProgDir
				WHEN CheckDirs = 2 THEN
					CheckDir = CurDir
				WHEN CheckDirs = 3 THEN
					CheckDir = Value('HOME', , 'OS2ENVIRONMENT')
				WHEN CheckDirs = 4 THEN
					CheckDir = Value('ETC', , 'OS2ENVIRONMENT')
				OTHERWISE
					LEAVE
			END

			IF Right(NewDir, 1) = '\' THEN
				CheckDir = Left(CheckDir, Length(NewDir) - 1)

			IF Stream(CheckDir||'\'||SBConf, 'c', 'query exists') = '' THEN
				CheckDirs = CheckDirs + 1
			ELSE
				DO
					SBConf = CheckDir||'\'||SBConf
					Found = 1
				END
		END

	IF Found <> 1 THEN
		DO UNTIL Stream(SBConf, 'c', 'query exists') <> ''
			SAY 'Could not find' SBConf||'.'
			SAY 'Enter the full path and filename of the configuration file to use.'
			PARSE PULL SBConf
		END
	ELSE
		DO
			IF (SBConfGiven < 2) & (CheckDir <> ProgDir) & (CheckDir <> CurDir) THEN
				DO
					SAY 'I found the configuration file at' SBConf
					SAY 'Do you wish to use this configuration file? (Y/N)'
					DO UNTIL Junk = 'Y' | Junk = 'N'
						Junk = Translate(SysGetKey(NOECHO))
					END
					IF Junk = 'N' THEN
						DO UNTIL Stream(SBConf, 'c', 'query exists') <> ''
							SAY 'Enter the full path and filename of the configuration file to use.'
							PARSE PULL SBConf
						END
				END
		END

	/* Read global configuration file into variables */
	RC = Stream(SBConf, 'c', 'open read')
	DO WHILE Lines(SBConf) > 0
		InLine = Linein(SBConf, , 1)
		IF Left(InLine, 2) = '/*' | Left(InLine, 2) = '*/' | InLine = '' THEN
			ITERATE
		ELSE
			SELECT
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'DISCSIZE' THEN
					DiscSize = Substr(InLine, Pos('=', InLine) + 1)
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'UNIT' THEN
					Unit = Substr(InLine, Pos('=', InLine) + 1)
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'RECURSE' THEN
					Recurse = Substr(InLine, Pos('=', InLine) + 1)
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'EMPTYDIRS' THEN
					EmptyDirs = Substr(InLine, Pos('=', InLine) + 1)
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'MINSIZE' THEN
					MinSize = Substr(InLine, Pos('=', InLine) + 1)
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'MAXSIZE' THEN
					MaxSize = Substr(InLine, Pos('=', InLine) + 1)
				WHEN Translate(Left(InLine, Pos('=', InLine) - 1)) = 'ALLOCATIONUNIT' THEN
					AllocationUnit = Substr(InLine, Pos('=', InLine) + 1)
				OTHERWISE
					NOP
			END
	END
	RC = Stream(SBConf, 'c', 'close')
RETURN


DiminishList:
	SAY 'Checking for files in the exclusions list...'
	RC = Stream(ExcludesFile, 'c', 'open')
	DO WHILE Lines(ExcludesFile) > 0
		Exclude = Linein(ExcludesFile, , 1)

		DO Num = 1 TO File.0
			PARSE VAR File.Num Date Time Size Attribs Filename
			SELECT
				WHEN Left(Exclude, 5) = 'FILE' THEN
					DO
						Exclude = Substr(Exclude, 6)
						Filename = Filespec('n', Filename)
					END
				WHEN Left(Exclude, 5) = 'PATH' THEN
					DO
						Exclude = Substr(Exclude, 6)
						Filename = Filespec('p', Filename)
					END
				OTHERWISE
					NOP
			END

			/* Call wildcard matching procedure */
			IF StringMatch(Filename, Exclude) = 1 THEN
				DO
					RC = SysStemDelete('File.', Num)
					Num = Num - 1
				END
		END
	END
	RC = Stream(ExcludesFile, 'c', 'close')

	/* This section allows the user to include empty sub-directories */
	IF Translate(EmptyDirs) = 'YES' THEN
		DO ThisDirNum = 1 TO DirNum
			ThisDir = InputDir.ThisDirNum

			RC = SysFileTree(ThisDir||'\', 'Dir', 'DS')

			IF DataType(Dir.0) = 'NUM' & Dir.0 > 0 THEN
				DO
					X = 0
					DO UNTIL X = Dir.0
						X = X + 1
						PARSE VAR Dir.X Date Time Size Attribs DirName
						DO Y = 1 TO File.0
							PARSE VAR File.Y Date Time Size Attribs Filename
							Filename = Filespec('d', Filename)||Filespec('p', Filename)
							IF Pos(DirName||'\', Filename) > 0 THEN
								DO
									RC = SysStemDelete('Dir.', X)
									X = X - 1
									LEAVE
								END
						END
					END
					DO Z = 1 TO Dir.0
						NewFile = File.0 + Z
						File.NewFile = Dir.Z
					END
					File.0 = File.0 + Dir.0
				END
		END
RETURN


RoundRobin:
	DO Num = 1 TO File.0
		PARSE VAR File.Num Date Time Size Attribs Filename

		IF (Size / AllocationUnit > Size % AllocationUnit) THEN
			Size = ((Size % AllocationUnit) + 1) * AllocationUnit
		Size = Size / 1024**Precision
		MinAlloc = 2 / 1024**(Precision - 1)
		IF Size < MinAlloc THEN
			Size = MinAlloc

		Size = Strip(Format(Size, NumBefore, NumAfter))

		Filename = Strip(Filename)

		FoundSpace = 0
		IF Project.0 = 0 THEN
			CALL MakeProject

		/* If current file is larger than disc size, byte split it and add to consecutive project folders */
		IF Size > DiscSize THEN
			DO
				SAY LRed||Filename||Non 'is larger than' DiscSize Unit '('||Size Unit||')'
				SAY 'File will be byte split to fit consecutive discs.'

				CALL ClearQ
				'@bytesplit.cmd -TS -C' DiscSize '-U' Unit '-I "'||Filename||'" | RXQUEUE'

				PULL Line
				NumChunks = Strip(Substr(Line, LastPos(' ', Line)), 'B', ' ')

				SizeRemain = Size
				DO ChunkNum = 1 to NumChunks
					FoundSpace = 0
					ThisFilename = Filename||'.'||Right(ChunkNum, 3, '0')
					IF SizeRemain > DiscSize THEN
						DO
							SizeRemain = SizeRemain - DiscSize
							ThisSize = DiscSize
						END
					ELSE
						ThisSize = SizeRemain
					CALL CheckSpace
				END
			END
		ELSE
			DO
				ThisSize = Size
				ThisFilename = Filename
				CALL CheckSpace
			END
	END
RETURN


CheckSpace:
	/* Use a "best fit" bin sort where files are added to the existing project list which has the LEAST free space */
	/* which can hold that file.  If no such project exists, create one. */
	DO X = 1 TO Project.0
		SortProject.X = Project.X.FreeSize||'09'x||'Project.'||X
	END

	RC = SysStemSort(SortProject)

	DO Count = 1 TO Project.0
		FreeSpace = Left(SortProject.Count, Pos('09'x, SortProject.Count) - 1)
		IF ThisSize <= FreeSpace THEN
			DO
				ThisProject = Substr(SortProject.Count, LastPos('.', SortProject.Count) + 1)
				Project.ThisProject.FreeSize = FreeSpace - ThisSize
				FoundSpace = 1
			END
		IF FoundSpace = 1 THEN
			LEAVE
	END

	DROP SortProject

	IF FoundSpace <> 1 THEN
		DO
			CALL MakeProject
			Count = Project.0
			Project.Count.FreeSize = Project.Count.FreeSize - ThisSize
		END

	CALL Lineout TempDir||'\sb_project.'||Count, ThisSize||'09'x||Unit||'09'x||ThisFilename
RETURN


MakeProject:
	/* We need to make a new project folder */
	Projects = Project.0 + 1
	Project.0 = Projects
	Project.Projects.FreeSize = DiscSize

	ProjectFile = TempDir||'\sb_project.'||Projects

	/* If a file already exists with this name, delete it to avoid potential problems later */
	RC = Stream(ProjectFile, 'c', 'query exists')
	IF RC <> '' THEN
		DO
			RC = SysFileDelete(ProjectFile)
			IF RC <> 0 THEN
				DO
					SAY 'Unable to delete existing file' ProjectFile '(rc =' RC||').'
					SAY 'I will now exit.  Please manually delete' ProjectFile 'and run me again.'
					EXIT
				END
		END
	RC = Stream(ProjectFile, 'c', 'open')
	CALL Lineout ProjectFile, 'SBConf='||SBConf
RETURN


PostProcess:
	NewDir = Directory(ProgDir)
	SELECT
		WHEN PostProcess <> '' & Translate(PostProcess) <> 'NONE' THEN
			INTERPRET 'CALL' PostProcess Project.0
		WHEN PostProcess = '' THEN
			DO
				RC = SysFileTree(ProgDir||'\pp_*.cmd', 'PPFile', 'F')
				SAY 'You will now need to select a post-processing script.  Available scripts are:'
				DO Y = 1 TO PPFile.0
					PARSE VAR PPFile.Y Date Time Size Attribs Filename
					PPFile.Y = Filespec('n', Filename)
					SAY Y||'09'x||PPFile.Y
				END
				SAY Y||'09'x||'Quit without post-processing'
				SAY 'For explanations of what each script does, consult the header of the relevant'
				SAY '.cmd file.'
				DO UNTIL Selection >= 1 & Selection <= Y
					Selection = SysGetKey('NOECHO')
				END
				IF Selection < Y THEN
					INTERPRET 'CALL' PPFile.Selection Project.0
				ELSE
					RETURN
			END
		OTHERWISE
			NOP
	END
	NewDir = Directory(Curdir)
RETURN


StringMatch: PROCEDURE
	/**********************************************************************/
	/* This wildcard matching procedure is based on work by Marcel Mller */
	/* and has been copied with permission.  Thank you!                   */
	/**********************************************************************/
	String = Translate(ARG(1))
	Template = Translate(ARG(2))
redo:
	IF Length(String) = 0 THEN
		RETURN Verify(Template, '*') = 0 /* '' matches only if template consists only of '*' */

	P = Verify(Template, '?*', 'M')

	IF P = 0 THEN
		RETURN Compare(String, Template, '*') = 0 /* no wildcards => compare strings */

	IF Compare(Left(String, P - 1), Left(Template, P - 1, '*')) \= 0 THEN
		RETURN 0 /* compare of non-wildcard section failed */

	J = P

	DO I = P TO Verify(Template' ', '*?',, P + 1) - 1 /* count # of ? */
		IF Substr(Template, I, 1) = '?' THEN
			J = J + 1
	END

	IF J > Length(String) + 1 THEN
		RETURN 0 /* more ? than length of str */
	Template = Substr(Template, I)
	String = Substr(String, J)
	IF I = J THEN
		SIGNAL redo

	/* '*' */
	IF Length(Template) = 0 THEN
		RETURN 1 /* nothing after '*' */
	DO P = 1
		P = Pos(Left(Template, 1), String, P)

		IF P = 0 THEN
			RETURN 0 /* character after '*' not found */

		IF StringMatch(Substr(String, P + 1), Substr(Template, 2)) THEN
			RETURN 1 /* got it! */
	END
RETURN


ClearQ:
	/* Clear anything currently on the RxQueue */
	DO WHILE Queued() > 0
		PULL Junk
	END
RETURN


NOTREADY:
	SAY 'A file I/O error has occured on line:' Sigl
	SAY 'Program statement is:' Sourceline(Sigl)
	SAY 'Program terminated.'
EXIT


HALT:
SAY 'Do you really wish to quit? (Y/N)'
Junk = Translate(SysGetKey(NOECHO))
IF Junk = 'Y' THEN
	EXIT
RETURN

